#!/bin/bash
#
# Copyright 2015 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

### Define image name defaults
## example: resultimage_name=univention/ucs-member-amd64:4.0-0
docker_repository_user="univention"
docker_image_prefix="ucs"
docker_image_configuration=''	# e.g. "member"		## derived below
docker_image_ucsversion=''	# e.g. "4.0-0"	## derived below
docker_image_pkg_arch=$(/usr/bin/dpkg --print-architecture)
docker_image_tag="rev0"

# First ensure docker.io is installed

is_package_installed ()
{
	local package="$1"
	case "$(dpkg-query -W -f '${Status}' $package 2>/dev/null)" in
		install*) return 0;;
		*) return 1 ;;
    esac
}

if ! is_package_installed docker.io; then
	ucr set repository/online/component/docker=enabled \
			repository/online/component/docker/unmaintained=enabled
	univention-install -y docker.io cgroupfs-mount aufs-tools
fi

image_name_is_available() {
	local image_name
	local image_uuid
	image_name="$1"
	image_uuid=$(docker inspect --format='{{.Id}}' "$image_name" 2>/dev/null || true)
	if [ -n "$image_uuid" ]; then
		echo -e "ERROR: Docker image name $image_name already exists. UUID: $image_uuid"
		echo -e "ERROR: That name cannot be assigned to the new image."
		echo -e "INFO:  Consider assigning an alternative tag to the existing image:"
		echo -e "INFO:  \tdocker tag $image_name othername1"
		echo -e "INFO:  After that you may free the name my running:"
		echo -e "INFO:  \tdocker rmi $image_name"
		return 1
	fi
	return 0
}

argv0="$0"
usage() {
	echo >&2 "usage: $argv0 [-i <result image name>] <UCS stem image> <univention-role>"
	echo >&2 "e.g.:  $argv0 univention/ucs-stem-amd64:4.0-1 member"
	exit 1
}

if [ $# -lt 1 ]; then
	usage
fi

opttemp=$(getopt --options '+c:e:i:t:v:' --longoptions 'configuration:,debug,env:,image:,resultimagename:,tag:,ucsversion:,help' --name "$argv0" -- "$@")
eval set -- "$opttemp"
unset opttemp

unset debug
unset resultimage_name
unset container_nameserver1
docker_run_options=( '-d' )
while true; do
	case "$1" in
		-c|--configuration)
			docker_image_configuration="$2";
			shift 2;
			;;
		--debug)
			debug=1
			shift;
			;;
		-e|--env)
			docker_run_options+=("-e" "$2");
			_testval="${2#nameserver1=}"
			if [ "$_testval" != "$2" ]; then
				container_nameserver1="$_testval"
			fi
			shift 2;
			;;
		-i|--image|--resultimagename)
			resultimage_name="$2";
			if ! image_name_is_available "$resultimage_name"; then
				exit 1
			fi
			shift 2;
			;;
		-t|--tag)
			docker_image_tag="$2";
			shift 2;
			;;
		-v|--ucsversion)
			docker_image_ucsversion="$2";
			shift 2;
			;;
		--help)
			usage;
			;;
		--)
			shift;
			break
			;;
		*)
			usage;
			;;
	esac
done

if [ -n "$debug" ]; then
	set -x
fi

stem_image_name="$1"
if [ -n "$stem_image_name" ]; then
	_testval=$(docker inspect -f '{{.Image}}' "$stem_image_name" 2>/dev/null)
	if [ "$?" -ne 0 ]; then
		echo "ERROR: Image name '$stem_image_name' not found"
		exit 1
	fi
	if [ "$_testval" != "<no value>" ]; then
		## Containers refer to an "Image":
		echo "ERROR: $stem_image_name is not an image, maybe it's a container."
		exit 1
	fi
	shift
else
	usage
fi
	
if [ -z "$resultimage_name" ] && [ -z "$docker_image_ucsversion" ]; then
	case "$stem_image_name" in
		univention/ucs-stem-amd64:4.0-0|univention/ucs-stem-i386:4.0-0)
			docker_image_ucsversion="4.0-0"
			;;
		univention/ucs-stem-amd64:4.0-1|univention/ucs-stem-i386:4.0-1)
			docker_image_ucsversion="4.0-1"
			;;
		univention/ucs-stem-amd64:4.0-2|univention/ucs-stem-i386:4.0-2)
			docker_image_ucsversion="4.0-2"
			;;
		*)
			echo "ERROR: unknown UCS stem image";
			echo "INFO: try passing --ucsversion" 
			echo "INFO: or --resultimagename" 
			usage;
			;;
	esac
fi

ucr_settings=()
system_role="$1"
install_packages=("univention-container-role-server-common")
case "$system_role" in
	master|server-master)
		system_role="master"
		ucr_settings+=("server/role=domaincontroller_master")
		install_packages+=("univention-server-master")
		shift;
		;;
	backup|server-backup)
		system_role="backup"
		ucr_settings+=("server/role=domaincontroller_backup")
		install_packages+=("univention-server-backup")
		shift;
		;;
	slave|server-slave)
		system_role="slave"
		ucr_settings+=("server/role=domaincontroller_slave")
		install_packages+=("univention-server-slave")
		shift;
		;;
	member|memberserver|server-member)
		system_role="member"
		ucr_settings+=("server/role=memberserver")
		install_packages+=("univention-server-member")
		shift;
		;;
	basesystem)
		system_role="basesystem"
		install_packages=("univention-container-basesystem")	## only this
		shift;
		;;
	*)
		echo "ERROR: unknown system role";
		usage;
		;;
esac

if [ -z "$docker_image_configuration" ]; then
	docker_image_configuration="$system_role"
fi

if [ -z "$resultimage_name" ]; then
	docker_image_basename="${docker_image_prefix}-${docker_image_configuration}-${docker_image_pkg_arch}"
	resultimage_name="${docker_repository_user}/${docker_image_basename}:${docker_image_ucsversion}"
fi

if ! image_name_is_available "$resultimage_name"; then
	exit 1
fi

if [ -z "$container_nameserver1" ]; then
	## inherit host values by default
	for var in nameserver1 nameserver2 nameserver3; do
		_testval=$(univention-config-registry get "$var")
		if [ -n "$_testval" ]; then
			echo "INFO: Configuring container $var=$_testval as on this host"
			docker_run_options+=("-e" "$var=$_testval")
		fi
	done
fi

uuid=$(docker run \
	"${docker_run_options[@]}" \
	"$stem_image_name" \
	/sbin/init)

## Wait for ((UCR committed resolv.conf) and (runlevel 2))
echo -n "INFO: Waiting for container runlevel 2 "
for ((i=0;i<30;i++)); do
	sleep 1	## first wait, the /sbin/init wrapper needs to reset utmp
	echo -n "."
	read clvl rlvl < <(docker exec "$uuid" runlevel)
	if [ "$rlvl" = 2 ]; then
		break 2
	fi
done
if [ "$i" -eq 30 ]; then
	echo "Timeout"
	exit 1
else
	echo "Ok"
fi

## Univention: Install role package
export DEBIAN_FRONTEND=noninteractive

ucr_settings+=(
repository/online=true
repository/online/server=univention-repository.knut.univention.de
repository/online/component/ucs-container=enabled
repository/online/component/ucs-container/unmaintained=yes
system/setup/boot/pkgcache=false
system/setup/boot/start=false
)

docker exec "$uuid" /usr/sbin/univention-config-registry set "${ucr_settings[@]}"

ucs_apt_get_options=(-o DPkg::Options::=--force-confold -o DPkg::Options::=--force-overwrite -o DPkg::Options::=--force-overwrite-dir --trivial-only=no --assume-yes --quiet=1)

## Note syntax: ${install_packages[*]} is required instead of ${install_packages[@]}
##              to keep bash/dash from breaking up the surrounding quotes:
docker exec "$uuid" bash -c "apt-get update \
	&& apt-get "${ucs_apt_get_options[@]}" install univention-docker-container-mode \
	&& apt-get "${ucs_apt_get_options[@]}" install ${install_packages[*]} \
	&& apt-get "${ucs_apt_get_options[@]}" install --no-install-recommends univention-system-setup-boot univention-management-console \
	&& apt-get "${ucs_apt_get_options[@]}" install smbclient curl"

## additionally hide the role dialog in system setup and set locale for system setup:
docker exec "$uuid" /usr/sbin/univention-config-registry set \
	system/setup/boot/pages/blacklist="role SoftwarePage" \
	locale="de_DE.UTF-8:UTF-8 en_US.UTF-8:UTF-8"

## Univention: Cleanup: remove policy-rc.d blocker and clean apt stuff
docker exec "$uuid" apt-get autoremove -y
docker exec "$uuid" ucr set repository/online/server=updates.software-univention.de
docker exec "$uuid" apt-get update
docker exec "$uuid" bash -c "apt-get clean && find /var/lib/apt/lists -type f -exec rm {} +"

#### Next step: Commit new Docker image

## Halt the container processes
_tmpval=$(docker stop "$uuid")
## Suppress useless default output
if [ "$_tmpval" != "$uuid" ]; then
	echo "$_tmpval"
fi

docker commit "$uuid" "$resultimage_name" || exit 1

## Remove the work container if not in debug mode
if [ -z "$debug" ]; then
	_tmpval=$(docker rm "$uuid")
	## Suppress useless default output
	if [ "$_tmpval" != "$uuid" ]; then
		echo "$_tmpval"
	fi
else
	echo -e "Now you may remove the temporary container by running:\n\tdocker rm '$uuid'"
fi

resultimage_basename="${resultimage_name%:*}"
resultimage_name_latest="$resultimage_basename:latest"
if [ "$resultimage_name_latest" != "$resultimage_name" ]; then
	if image_name_is_available "$resultimage_name_latest"; then
		docker tag "$resultimage_name" "$resultimage_name_latest"
	else
		echo 'This should be fixed before uploading!'
	fi
fi

echo -e "The $system_role role image has been generated: $resultimage_name"
