#!/bin/bash -e
#
# Copyright 2015 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

### Define image name defaults
## example: resultimage_name=univention/ucs-stem-amd64:4.0-0
docker_repository_user="univention"
docker_image_prefix="ucs"
docker_image_configuration="stem"
docker_image_ucsversion=''	# e.g. "4.0-0"	## derived below
docker_image_pkg_arch=$(/usr/bin/dpkg --print-architecture)
docker_image_tag="rev0"

# First ensure docker.io is installed

is_package_installed ()
{
	local package="$1"
	case "$(dpkg-query -W -f '${Status}' $package 2>/dev/null)" in
		install*) return 0;;
		*) return 1 ;;
    esac
}

if ! is_package_installed docker.io; then
	ucr set repository/online/component/docker=enabled \
			repository/online/component/docker/unmaintained=enabled
	univention-install -y docker.io cgroupfs-mount aufs-tools
fi

image_name_is_available() {
	local image_name
	local image_uuid
	image_name="$1"
	image_uuid=$(docker inspect --format='{{.Id}}' "$image_name" 2>/dev/null || true)
	if [ -n "$image_uuid" ]; then
		echo -e "ERROR: Docker image name $image_name already exists. UUID: $image_uuid"
		echo -e "ERROR: That name cannot be assigned to the new image."
		echo -e "INFO:  Consider assigning an alternative tag to the existing image:"
		echo -e "INFO:  \tdocker tag $image_name othername1"
		echo -e "INFO:  After that you may free the name my running:"
		echo -e "INFO:  \tdocker rmi $image_name"
		return 1
	fi
	return 0
}

argv0="$0"
usage() {
	echo >&2 "usage: $argv0 <Repository-URL>"
	echo >&2 "e.g.:  $argv0 http://updates.software-univention.de/4.0/maintained/4.0-2/"
	exit 1
}

if [ $# -lt 1 ]; then
	usage
fi

opttemp=$(getopt --options '+c:i:t:v:' --longoptions 'configuration:,image:,resultimagename:,tag:,ucsversion:,help' --name "$argv0" -- "$@")
eval set -- "$opttemp"
unset opttemp

unset dist
unset resultimage_name
while true; do
	case "$1" in
		-c|--configuration)
			docker_image_configuration="$2";
			shift 2;
			;;
		-i|--image|--resultimagename)
			resultimage_name="$2";
			if ! image_name_is_available "$resultimage_name"; then
				exit 1
			fi
			shift 2;
			;;
		-t|--tag)
			docker_image_tag="$2";
			shift 2;
			;;
		-v|--ucsversion)
			docker_image_ucsversion="$2";
			shift 2;
			;;
		--help)
			usage;
			;;
		--)
			shift;
			break
			;;
		*)
			usage;
			;;
	esac
done

ucs_repo_url="$1"
repo_scope=$(basename "$ucs_repo_url")
case "$repo_scope" in
	"4.0-0"|"ucs_4.0-0")
		dist=ucs400;
		if [ -z "$docker_image_ucsversion" ]; then
			docker_image_ucsversion="4.0-0"
		fi
		shift;
		;;
	"4.0-1"|"ucs_4.0-0-ucs4.0-1")
		dist=ucs401;
		if [ -z "$docker_image_ucsversion" ]; then
			docker_image_ucsversion="4.0-1"
		fi
		shift;
		;;
	"4.0-2"|"ucs_4.0-0-ucs4.0-2")
		dist=ucs402;
		if [ -z "$docker_image_ucsversion" ]; then
			docker_image_ucsversion="4.0-2"
		fi
		shift;
		;;
	*)
		echo "ERROR: unknown UCS version";
		usage;
		;;
esac

if [ -z "$resultimage_name" ]; then
	docker_image_basename="${docker_image_prefix}-${docker_image_configuration}-${docker_image_pkg_arch}"
	resultimage_name="${docker_repository_user}/${docker_image_basename}:${docker_image_ucsversion}"
fi

if ! image_name_is_available "$resultimage_name"; then
	exit 1
fi

# Create a minimal UCS $dist image

## Install haveged to avoid delays due to entropy depeletion
if ! is_package_installed haveged; then
	_unmaintained_setting=$(ucr get repository/online/unmaintained)
	ucr set repository/online/unmaintained="yes"
	univention-install -y haveged
	ucr set repository/online/unmaintained="$_unmaintained_setting"
fi

unset temp_bootstrap_dir
if [ -n "$DOCKER_BOOTSTRAP_DIR" ]; then
	[ -d "$DOCKER_BOOTSTRAP_DIR" ] || mkdir -p "$DOCKER_BOOTSTRAP_DIR"
	rootfsdir="$DOCKER_BOOTSTRAP_DIR"
else
	temp_bootstrap_dir="$(mktemp -d ${TMPDIR:-/var/tmp}/docker-baseimage.XXXXXXXXXX)"
	rootfsdir="$temp_bootstrap_dir/rootfs"

	cleanup () {
		umount "$rootfsdir"/proc || true
		echo "Removing temporary $temp_bootstrap_dir"
		rm -rf "$temp_bootstrap_dir"
	}
	trap cleanup EXIT

	mkdir -p "$rootfsdir" && echo "Created temporary $rootfsdir"
fi

debootstrap --exclude=tasksel,plymouth "$dist" \
	"$rootfsdir" \
	"$ucs_repo_url" \
	/usr/share/debootstrap/scripts/wheezy || exit 1

## copied from /usr/share/docker.io/contrib/mkimage/debootstrap :
if [ -d "$rootfsdir/etc/apt/apt.conf.d" ]; then
	# _keep_ us lean by effectively running "apt-get clean" after every install
	aptGetClean='"rm -f /var/cache/apt/archives/*.deb /var/cache/apt/archives/partial/*.deb /var/cache/apt/*.bin || true";'
	echo >&2 "+ cat > '$rootfsdir/etc/apt/apt.conf.d/docker-clean'"
	cat > "$rootfsdir/etc/apt/apt.conf.d/docker-clean" <<-EOF
		# Since for most Docker users, package installs happen in "docker build" steps,
		# they essentially become individual layers due to the way Docker handles
		# layering, especially using CoW filesystems.  What this means for us is that
		# the caches that APT keeps end up just wasting space in those layers, making
		# our layers unnecessarily large (especially since we'll normally never use
		# these caches again and will instead just "docker build" again and make a brand
		# new image).

		# Ideally, these would just be invoking "apt-get clean", but in our testing,
		# that ended up being cyclic and we got stuck on APT's lock, so we get this fun
		# creation that's essentially just "apt-get clean".
		DPkg::Post-Invoke { ${aptGetClean} };
		APT::Update::Post-Invoke { ${aptGetClean} };

		Dir::Cache::pkgcache "";
		Dir::Cache::srcpkgcache "";

		# Note that we do realize this isn't the ideal way to do this, and are always
		# open to better suggestions (https://github.com/docker/docker/issues).
	EOF

	# remove apt-cache translations for fast "apt-get update"
	echo >&2 "+ echo Acquire::Languages 'none' > '$rootfsdir/etc/apt/apt.conf.d/docker-no-languages'"
	cat > "$rootfsdir/etc/apt/apt.conf.d/docker-no-languages" <<-'EOF'
	# In Docker, we don't often need the "Translations" files, so we're just wasting
	# time and space by downloading them, and this inhibits that.  For users that do
	# need them, it's a simple matter to delete this file and "apt-get update". :)

	Acquire::Languages "none";
	EOF

	echo >&2 "+ echo Acquire::GzipIndexes 'true' > '$rootfsdir/etc/apt/apt.conf.d/docker-gzip-indexes'"
	cat > "$rootfsdir/etc/apt/apt.conf.d/docker-gzip-indexes" <<-'EOF'
	# Since Docker users using "RUN apt-get update && apt-get install -y ..." in
	# their Dockerfiles don't go delete the lists files afterwards, we want them to
	# be as small as possible on-disk, so we explicitly request "gz" versions and
	# tell Apt to keep them gzipped on-disk.

	# For comparison, an "apt-get update" layer without this on a pristine
	# "debian:wheezy" base image was "29.88 MB", where with this it was only
	# "8.273 MB".

	Acquire::GzipIndexes "true";
	Acquire::CompressionTypes::Order:: "gz";
	EOF
fi

## prevent init scripts from running during install/update
cat > "$rootfsdir/usr/sbin/policy-rc.d" <<'EOF'
#!/bin/sh

# For most Docker users, "apt-get install" only happens during "docker build",
# where starting services doesn't work and often fails in humorous ways. This
# prevents those failures by stopping the services from attempting to start.

exit 101
EOF
chmod +x "$rootfsdir/usr/sbin/policy-rc.d"

## Univention: Mount /proc for postinst scripts
# chroot "$rootfsdir" mount -t proc proc /proc
mount -o bind /proc "$rootfsdir"/proc

## Univention: Install only univention-updater
export DEBIAN_FRONTEND=noninteractive

ucr_settings=(
)

## inherit host values by default
for var in nameserver1 nameserver2 nameserver3; do
	_testval=$(univention-config-registry get "$var")
	if [ -n "$_testval" ]; then
		echo "INFO: Configuring container $var=$_testval as on this host"
		ucr_settings+=("$var=$_testval")
	fi
done

if [ "${#ucr_settings[@]}" -gt 0 ]; then
	chroot "$rootfsdir" ucr set "${ucr_settings[@]}"
fi

ucs_apt_get_options=(-o DPkg::Options::=--force-confold -o DPkg::Options::=--force-overwrite -o DPkg::Options::=--force-overwrite-dir --trivial-only=no --assume-yes --quiet=1)

chroot "$rootfsdir" apt-get update \
	&& chroot "$rootfsdir" apt-get "${ucs_apt_get_options[@]}" install univention-updater

chroot "$rootfsdir" ucr set repository/online=true \
	repository/online/unmaintained=yes \
	repository/online/server=univention-repository.knut.univention.de \
	repository/online/component/docker=enabled \
	repository/online/component/docker/unmaintained=yes \
	repository/online/component/ucs-container=enabled \
	repository/online/component/ucs-container/unmaintained=yes \
	system/setup/boot/pkgcache=false \
	system/setup/boot/start=false

echo "">"$rootfsdir/etc/apt/sources.list"

## hide some dialogs in system setup and set locale for system setup:
chroot "$rootfsdir" /usr/sbin/univention-config-registry set \
	system/setup/boot/installer=true \
	system/setup/boot/pages/blacklist="SoftwarePage" \
	system/setup/boot/fields/blacklist='network password hostname' \
	locale="de_DE.UTF-8:UTF-8 en_US.UTF-8:UTF-8"

chroot "$rootfsdir" apt-get update \
	&& chroot "$rootfsdir" apt-get "${ucs_apt_get_options[@]}" -u dist-upgrade \
	&& chroot "$rootfsdir" apt-get "${ucs_apt_get_options[@]}" install univention-ssl univention-apache \
	&& chroot "$rootfsdir" apt-get "${ucs_apt_get_options[@]}" install univention-docker-container-mode \
	&& chroot "$rootfsdir" apt-get "${ucs_apt_get_options[@]}" install --no-install-recommends univention-system-setup-boot univention-management-console \
	&& chroot "$rootfsdir" apt-get "${ucs_apt_get_options[@]}" install smbclient curl

## Univention: remove policy-rc.d blocker and clean apt stuff
chroot "$rootfsdir" apt-get autoremove -y
chroot "$rootfsdir" ucr set repository/online/unmaintained=no \
	repository/online/server=updates.software-univention.de
chroot "$rootfsdir" apt-get update

rm "$rootfsdir/usr/sbin/policy-rc.d"
chroot "$rootfsdir" apt-get clean && chroot "$rootfsdir" find /var/lib/apt/lists -type f -exec rm {} +

umount "$rootfsdir"/proc


#### Next step: Build a Docker image

echo "Importing image into docker as \"$resultimage_name\" ..."
tar -C "$rootfsdir" -c . | docker import - "$resultimage_name"

if [ -n "$temp_bootstrap_dir" ]; then
	trap true EXIT
	echo "Removing temporary $temp_bootstrap_dir"
	rm -rf "$temp_bootstrap_dir"
fi

resultimage_basename="${resultimage_name%:*}"
resultimage_name_latest="$resultimage_basename:latest"
if [ "$resultimage_name_latest" != "$resultimage_name" ]; then
	if image_name_is_available "$resultimage_name_latest"; then
		docker tag "$resultimage_name" "$resultimage_name_latest"
	else
		echo 'This should be fixed before uploading!'
	fi
fi

echo "Stem image '$resultimage_name' built. Now UCS role images may be derived from this."
