#!/bin/bash -e
#
# Copyright 2015 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

# Instanciate and join a docker app container from an app image

argv0="$0"
usage() {
	echo >&2 "usage: $argv0 -h <container hostname> [--name <container name>] <app image name> --joinuser <joinuser> --joinpassword <joinpassword>"
	exit 1
}

opttemp=$(getopt --options '+a:e:h:p:v:' --longoptions 'appname:,env:,hostname:,joinpassword:,joinuser:,name:,containername:,volume:,help' --name "$argv0" -- "$@")
eval set -- "$opttemp"
unset opttemp

unset container_hostname
unset container_domainname
unset container_nameserver1
unset container_rootpwd
unset app_name
docker_run_options=( -d )
while true; do
	case "$1" in
		-a|--appname)
			app_name="$2";
			shift 2;
			;;
		-e|--env)
			docker_run_options+=("-e" "$2");
			_testval="${2#domainname=}"
			if [ "$_testval" != "$2" ]; then
				container_domainname="$_testval"
			fi
			_testval="${2#nameserver1=}"
			if [ "$_testval" != "$2" ]; then
				container_nameserver1="$_testval"
			fi
			_testval="${2#rootpwd=}"
			if [ "$_testval" != "$2" ]; then
				container_rootpwd="$_testval"
			fi
			shift 2;
			;;
		-h|--hostname)
			docker_run_options+=("-h" "$2");
			container_hostname="$2";
			shift 2;
			;;
		-p)
			docker_run_options+=("-p" "$2");
			shift 2;
			;;
		--joinpassword)
			join_password="$2";
			shift 2;
			;;
		--joinuser)
			join_username="$2";
			shift 2;
			;;
		--name|--containername)
			docker_run_options+=("--name" "$2");
			shift 2;
			;;
		-v|--volume)
			docker_run_options+=("-v" "$2");
			shift 2;
			;;
		--help)
			usage;
			;;
		--)
			shift;
			break;
			;;
	esac
done

app_image_name="$1"
if [ -z "$app_image_name" ]; then
	usage
fi
shift

if [ -z "$container_hostname" ]; then
	usage
fi

unset using_host_domain
if [ -z "$container_domainname" ]; then
	## inherit host value by default
	_testval=$(univention-config-registry get domainname)
	if [ -n "$_testval" ]; then
		echo "Configuring container domainname=$_testval as on this host"
		docker_run_options+=("-e" "domainname=$_testval")
		using_host_domain=true
	fi
fi

if [ -n "$container_hostname" ] && [ -n "$using_host_domain" ]; then
	## On a UCS host we may check if the container hostname is already in LDAP
	if [ -x /usr/bin/univention-ldapsearch ]; then
		_testval=$(/usr/bin/univention-ldapsearch -xLLL \
			"(&(objectClass=univentionHost)(cn=$container_hostname))" dn)
		if [ -n "$_testval" ]; then
			echo "WARNING: The chosen hostname already exists in the LDAP of this host!"
			read -p "Continue anyway? [yN]"
			if [ "${REPLY^^}" != Y ]; then
				exit 1
			fi
		fi
	fi
fi

if [ -z "$container_nameserver1" ]; then
	## inherit host values by default
	for var in nameserver1 nameserver2 nameserver3; do
		_testval=$(univention-config-registry get "$var")
		if [ -n "$_testval" ]; then
			echo "Configuring container $var=$_testval as on this host"
			docker_run_options+=("-e" "$var=$_testval")
		fi
	done
fi

if [ -z "$join_username" ]; then
	usage
fi
shift

if [ -z "$join_password" ]; then
	usage
fi
shift

eval "$(univention-config-registry shell)"

## Derive the app name from the image name
if [ -z "$app_name" ]; then
	app_name="${app_image_name%:*}"
	app_name="${app_name#*/}"
fi

all_container_ids=($(docker ps -qa))
if [ -n "$all_container_ids" ]; then
	### Check that no other running container has been derived from the App-Image
	all_running_images=($(docker inspect --format='{{.Config.Image}}' "${all_container_ids[@]}"))
	for i in "${all_running_images[@]}"; do
		if [ "$i" = "$app_image_name" ]; then
			echo "ERROR: A container is already running this app image"
			exit 1
		fi
	done

	### Check that hostname is not used by another container

	all_container_hostnames=($(docker inspect --format='{{.Config.Hostname}}' "${all_container_ids[@]}"))
	for i in "${all_container_hostnames[@]}"; do
		if [ "$i" = "$container_hostname" ]; then
			echo "ERROR: A container is already using this hostname"
			exit 1
		fi
	done
fi

app_container_var_dir="/var/lib/appcenter/$app_name"
app_container_data_dir="$app_container_var_dir/data"
# app_container_config_dir="$app_container_var_dir/config"
# app_container_etc_dir="$app_container_config_dir/etc"
# app_container_etc_univention_dir="$app_container_etc_dir/univention"

## Not required, docker does this automatically:
## mkdir -p "$app_container_data_dir"
## mkdir -p "$app_container_config_dir"

### These next steps should already have been done by
### install-ucs-app-as-docker-image-layer.sh, but this script is
### supposed to be run on the customer host.. so we have to do it here as well
# mkdir -p "$app_container_etc_univention_dir"

## at least touch the files, otherwise docker creates them as directories:
# for fn in base.conf base-forced.conf base-ldap.conf base-schedule.conf; do
# 	touch "$app_container_etc_univention_dir/$fn"
# done
# touch "$app_container_etc_dir/machine.secret"

docker_run_options+=("-v" "$app_container_data_dir:$app_container_data_dir")

# -v "$app_container_config_dir/etc/univention/ssl:/etc/univention/ssl"
# -v "$app_container_config_dir/etc/machine.secret:/etc/machine.secret"
#-v "$app_container_config_dir/etc/univention/base.conf:/etc/univention/base.conf"
#-v "$app_container_config_dir/etc/univention/base-forced.conf:/etc/univention/base-forced.conf"
#-v "$app_container_config_dir/etc/univention/base-ldap.conf:/etc/univention/base-ldap.conf"
#-v "$app_container_config_dir/etc/univention/base-schedule.conf:/etc/univention/base-schedule.conf"
#-v "$app_container_config_dir:$app_container_config_dir"

if [ -z "$container_rootpwd" ]; then
	## Generate random root password and output below
	random_rootpwd=$(makepasswd --chars 8)
	docker_run_options+=("-e" "rootpwd=$random_rootpwd")
fi

## Boot the container
uuid=$(docker run \
	"${docker_run_options[@]}" \
	"$app_image_name" \
	/sbin/init)
echo "UUID: $uuid"

if [ -n "$container_rootpwd" ]; then
	salt="$(makepasswd --chars 8)"
	rootpwd_hash=$(mkpasswd -m sha-256 "$container_rootpwd" "$salt")
else
	## Get root password hash from host
	rootpwd_hash=$(sed -n 's/^root:\([^:]*\):.*/\1/p' /etc/shadow)
	if [ -z "$rootpwd_hash" ] || [ "$rootpwd_hash" = '*' ]; then
		## Generate random root password and output below
		random_rootpwd=$(makepasswd --chars 8)
		salt="$(makepasswd --chars 8)"
		rootpwd_hash=$(mkpasswd -m sha-256 "$random_rootpwd" "$salt")
	fi
fi

docker exec "$uuid" usermod -p "$rootpwd_hash" root

## Wait for ((UCR committed resolv.conf) and (runlevel 2))
echo -n "INFO: Waiting for container runlevel 2 "
for ((i=0;i<30;i++)); do
	sleep 1	## first wait, the /sbin/init wrapper needs to reset utmp
	echo -n "."
	read clvl rlvl < <(docker exec "$uuid" runlevel)
	if [ "$rlvl" = 2 ]; then
		break 2
	fi
done
if [ "$i" -eq 30 ]; then
	echo "Timeout"
	exit 1
else
	echo "Ok"
fi

## Inject the join password for univention-join
container_join_secret_file=$(docker exec "$uuid" mktemp)
docker exec -i "$uuid" bash -c "cat > '$container_join_secret_file'" <<<"$join_password"

## Join the container
docker exec -it "$uuid" \
	univention-join \
		-dcaccount "$join_username" \
		-dcpwd "$container_join_secret_file"
## Clean up
docker exec "$uuid" rm "$container_join_secret_file"

if [ -z "$container_rootpwd" ]; then
	if [ -n "$random_rootpwd" ]; then
		echo "INFO: Root password generated for container: $random_rootpwd"
	else
		echo "INFO: Root password hash copied from host to container"
	fi
fi

## show exposed ports:
docker_port_mapping=$(docker inspect -f '{{range $p, $conf := .NetworkSettings.Ports}}{{$p}}: {{(index $conf 0).HostPort}}{{"\n"}}{{end}}' "$uuid")
echo "INFO: exposed ports: ${docker_port_mapping//$'\n'/, }"
