#!/bin/bash -e
#
# Copyright 2015 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

# Instanciate an image as daemon
docker_image_pkg_arch=$(/usr/bin/dpkg --print-architecture)
argv0="$0"
usage() {
	echo >&2 "usage: $argv0 [-h <hostname>] [--name <containername>] [...] <imagename>"
	echo >&2 " e.g.: $argv0 --name myinstance1 univention/ucs-generic-$docker_image_pkg_arch:4.0-1"
	exit 1
}

opttemp=$(getopt --options '+h:e:p:v:' --longoptions 'containername:,hostname:,env:,name:,volume:,help' --name "$argv0" -- "$@")
eval set -- "$opttemp"
unset opttemp

unset container_hostname
unset container_domainname
unset container_nameserver1
unset container_rootpwd
unset container_name
docker_run_options=( '-d' )
while true; do
	case "$1" in
		-h|--hostname)
			docker_run_options+=("-h" "$2");
			container_hostname="$2";
			shift 2;
			;;
		-e|--env)
			docker_run_options+=("-e" "$2");
			_testval="${2#domainname=}"
			if [ "$_testval" != "$2" ]; then
				container_domainname="$_testval"
			fi
			_testval="${2#nameserver1=}"
			if [ "$_testval" != "$2" ]; then
				container_nameserver1="$_testval"
			fi
			_testval="${2#rootpwd=}"
			if [ "$_testval" != "$2" ]; then
				container_rootpwd="$_testval"
			fi
			shift 2;
			;;
		--name|--containername)
			docker_run_options+=("--name" "$2");
			container_name="$2";
			shift 2;
			;;
		-p)
			docker_run_options+=("-p" "$2");
			shift 2;
			;;
		-v|--volume)
			docker_run_options+=("-v" "$2");
			shift 2;
			;;
		--help)
			usage;
			;;
		--)
			shift;
			break;
			;;
	esac
done

imagename="$1"
if [ -z "$imagename" ]; then
	usage
fi

unset using_host_domain
if [ -z "$container_domainname" ]; then
	## inherit host value by default
	_testval=$(hostname -d)
	if [ -n "$_testval" ]; then
		echo "Configuring container domainname=$_testval as on this host"
		docker_run_options+=("-e" "domainname=$_testval")
		using_host_domain=true
	fi
fi

if [ -n "$container_hostname" ] && [ -n "$using_host_domain" ]; then
	## On a UCS host we may check if the container hostname is already in LDAP
	if [ -x /usr/bin/univention-ldapsearch ]; then
		_testval=$(/usr/bin/univention-ldapsearch -xLLL \
			"(&(objectClass=univentionHost)(cn=$container_hostname))" dn)
		if [ -n "$_testval" ]; then
			echo "WARNING: The chosen hostname already exists in the LDAP of this host!"
		fi
	fi
fi

if [ -z "$container_nameserver1" ]; then
	## inherit host values by default
	for var in nameserver1 nameserver2 nameserver3; do
		_testval=$(univention-config-registry get "$var")
		if [ -n "$_testval" ]; then
			echo "Configuring container $var=$_testval as on this host"
			docker_run_options+=("-e" "$var=$_testval")
		fi
	done
fi

uuid=$(docker run \
	"${docker_run_options[@]}" \
	"$imagename" \
	/sbin/init)
echo "UUID: $uuid"


## And now we can run additional commands in the container:
if [ -n "$container_rootpwd" ]; then
	salt="$(makepasswd --chars 8)"
	rootpwd_hash=$(mkpasswd -m sha-256 "$container_rootpwd" "$salt")
else
	## Get root password hash from host
	rootpwd_hash=$(sed -n 's/^root:\([^:]*\):.*/\1/p' /etc/shadow)
	if [ -z "$rootpwd_hash" ] || [ "$rootpwd_hash" = '*' ]; then
		## Generate random root password and output below
		random_rootpwd=$(makepasswd --chars 8)
		salt="$(makepasswd --chars 8)"
		rootpwd_hash=$(mkpasswd -m sha-256 "$random_rootpwd" "$salt")
	fi
fi

docker exec "$uuid" usermod -p "$rootpwd_hash" root

docker exec "$uuid" bash -c '. /etc/lsb-release; echo $DISTRIB_DESCRIPTION'

if [ -z "$container_rootpwd" ]; then
	if [ -n "$random_rootpwd" ]; then
		echo "INFO: Root password generated for container: $random_rootpwd"
	else
		echo "INFO: Root password hash copied from host to container"
	fi
fi

## show exposed ports:
docker_port_mapping=$(docker inspect -f '{{range $p, $conf := .NetworkSettings.Ports}}{{$p}}: {{(index $conf 0).HostPort}}{{"\n"}}{{end}}' "$uuid")
echo "INFO: exposed ports: ${docker_port_mapping//$'\n'/, }"

## Wait for ((UCR committed resolv.conf) and (runlevel 2))
echo -n "INFO: Waiting for container runlevel 2 "
for ((i=0;i<30;i++)); do
	sleep 1	## first wait, the /sbin/init wrapper needs to reset utmp
	echo -n "."
	read clvl rlvl < <(docker exec "$uuid" runlevel)
	if [ "$rlvl" = 2 ]; then
		break 2
	fi
done
if [ "$i" -eq 30 ]; then
	echo "Timeout"
	exit 1
else
	echo "Ok"
fi
