#!/bin/bash
#
# Copyright 2015 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

# Instanciate and setup-join a generic UCS Docker Container as a dummy domain
# The base image for this can be created with the generate-appliance script

docker_image_pkg_arch=$(/usr/bin/dpkg --print-architecture)
argv0="$0"
usage() {
	echo >&2 "usage: $argv0 -h <container hostname> [-i <result image name>] [--name <container name>] <imported image name>"
	exit 1
}

opttemp=$(getopt --options '+h:e:i:v:' --longoptions 'containername:,hostname:,env:,image:,name:,resultimagename:,volume:,help' --name "$argv0" -- "$@")
eval set -- "$opttemp"
unset opttemp

unset container_hostname
unset resultimage_name
docker_run_options=( '-d' )
while true; do
	case "$1" in
		-h|--hostname)
			docker_run_options+=("-h" "$2");
			container_hostname="$2";
			shift 2;
			;;
		-e|--env)
			docker_run_options+=("-e" "$2");
			shift 2;
			;;
		-i|--image|--resultimagename)
			resultimage_name="$2";
			shift 2;
			;;
		--name|--containername)
			docker_run_options+=("--name" "$2");
			shift 2;
			;;
		-v|--volume)
			docker_run_options+=("-v" "$2");
			shift 2;
			;;
		--help)
			usage;
			;;
		--)
			shift;
			break;
			;;
	esac
done

image_name="$1"
if [ -z "$image_name" ]; then
	usage
fi
shift

if [ -z "$container_hostname" ]; then
	usage
fi

## Boot the container
uuid=$(docker run \
	"${docker_run_options[@]}" \
	"$image_name" \
	/sbin/init)	## starts PID1 wrapper

if [ -n "$uuid" ]; then
	echo "UUID: $uuid"
else
	echo "Start of container failed"
	exit 1
fi

## Wait for ((UCR committed resolv.conf) and (runlevel 2))
echo -n "INFO: Waiting for container runlevel 2 "
for ((i=0;i<30;i++)); do
	sleep 1	## first wait, the /sbin/init wrapper needs to reset utmp
	echo -n "."
	read clvl rlvl < <(docker exec "$uuid" runlevel)
	if [ "$rlvl" = 2 ]; then
		break 2
	fi
done
if [ "$i" -eq 30 ]; then
	echo "Timeout"
	exit 1
else
	echo "Ok"
fi

## Install the customization package for appliance-docker-container
docker exec "$uuid" univention-install -y univention-appliance-docker-container

## workaround for Could not get lock /var/cache/apt/archives/lock - open (11: Resource temporarily unavailable):
sleep 1
docker exec "$uuid" touch /var/cache/univention-system-setup/packages/Packages
docker exec "$uuid" apt-get update
docker exec "$uuid" /usr/share/univention-docker-container-mode/download-packages \
		univention-container-role-server-common \
		univention-server-master
docker exec "$uuid" /usr/share/univention-docker-container-mode/download-packages \
		univention-container-role-server-common \
		univention-server-backup
docker exec "$uuid" /usr/share/univention-docker-container-mode/download-packages \
		univention-container-role-server-common \
		univention-server-slave
docker exec "$uuid" /usr/share/univention-docker-container-mode/download-packages \
		univention-container-role-server-common \
		univention-server-member
docker exec "$uuid" /usr/share/univention-docker-container-mode/download-packages \
		univention-basesystem

(
## get the UCR from the container
eval "$(docker exec "$uuid" univention-config-registry shell)"

## Inject the profile
docker exec -it "$uuid" bash -c 'cat > /var/cache/univention-system-setup/profile' <<%EOF
hostname="$hostname"
domainname="$domainname"
ldap/base="dc=${domainname//./,dc=}"
server/role="domaincontroller_master"
interfaces/eth0/type="$interfaces_eth0_type"
interfaces/eth0/address="$interfaces_eth0_address"
interfaces/eth0/network="$interfaces_eth0_network"
interfaces/eth0/broadcast="$interfaces_eth0_broadcast"
interfaces/eth0/netmask="$interfaces_eth0_netmask"
gateway="$gateway"
nameserver1="$nameserver1"
nameserver2="$nameserver2"
nameserver3="$nameserver3"
# organization="Univention Corporate Server"
ssl/organizationalunit="Univention Corporate Server"
ssl/organization="DE"
ssl/email="ssl@unassigned-domain"
ssl/state="DE"
ssl/locality="DE"
locale="de_DE.UTF-8:UTF-8 en_US.UTF-8:UTF-8"
locale/default="de_DE.UTF-8:UTF-8"
xorg/keyboard/options/XkbLayout="de"
timezone="Europe/Berlin"
components="[]"
packages_install=""
packages_remove=""
update/system/after/setup="False"
ad/member=""
%EOF
)

## Disable the /usr/lib/univention-system-setup/appliance-hooks.d for the dummy setup-join
docker exec -i "$uuid" bash -c "mv /usr/lib/univention-system-setup/appliance-hooks.d /usr/lib/univention-system-setup/appliance-hooks.d.disabled"

## Setup-join the container
docker exec -it "$uuid" \
	/usr/lib/univention-system-setup/scripts/setup-join.sh --dcaccount Administrator

## Move the /usr/lib/univention-system-setup/appliance-hooks.d back im place
docker exec -i "$uuid" bash -c "mv /usr/lib/univention-system-setup/appliance-hooks.d.disabled /usr/lib/univention-system-setup/appliance-hooks.d"

## Cleanup
docker exec "$uuid" apt-get clean
docker exec "$uuid" rm /root/.bash_history

if [ -n "$resultimage_name" ]; then
	docker stop "$uuid"
	docker commit "$uuid" "$resultimage_name"
	echo "A new container can now be started from this image by:"
	echo -e "\tdocker run -d -h unassigned-hostname-e domainname=univention.unconfigured $resultimage_name /sbin/init"
else
	echo "The container can now be stopped and commited to an image by running:"
	echo -e "\tdocker stop $uuid"
	echo -e "\tdocker commit $uuid univention/example-ucs-4.0-0-$docker_image_pkg_arch"
	echo
	echo "After that a new container can be started from that image by running:"
	echo -e "\tdocker run -d -h unassigned-hostname -e domainname=univention.unconfigured univention/example-ucs-4.0-0-$docker_image_pkg_arch /sbin/init"
fi
