#!/bin/bash
#
# Copyright 2015 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

# Run univention-upgrade on a generic UCS Docker Container to get a new layer

### Define image name defaults
## example: resultimage_name=univention/ucs-member-amd64:4.0-1
docker_repository_user="univention"
docker_image_prefix="ucs"

image_name_is_available() {
	local image_name
	local image_uuid
	image_name="$1"
	image_uuid=$(docker inspect --format='{{.Id}}' "$image_name" 2>/dev/null || true)
	if [ -n "$image_uuid" ]; then
		echo -e "ERROR: Docker image name $image_name already exists. UUID: $image_uuid"
		echo -e "ERROR: That name cannot be assigned to the new image."
		echo -e "INFO:  Consider assigning an alternative tag to the existing image:"
		echo -e "INFO:  \tdocker tag $image_name othername1"
		echo -e "INFO:  After that you may free the name my running:"
		echo -e "INFO:  \tdocker rmi $image_name"
		return 1
	fi
	return 0
}


docker_image_pkg_arch=$(/usr/bin/dpkg --print-architecture)
argv0="$0"
usage() {
	echo >&2 "usage: $argv0 [-i <result image name>] [--name <container name>] <imported image name>"
	exit 1
}

opttemp=$(getopt --options '+h:e:i:v:' --longoptions 'containername:,debug,hostname:,env:,image:,name:,resultimagename:,updateto:,volume:,help' --name "$argv0" -- "$@")
eval set -- "$opttemp"
unset opttemp

unset container_name
unset container_domainname
unset container_nameserver1
unset resultimage_name
unset updateto
unset debug
docker_run_options=( '-d' )
while true; do
	case "$1" in
		--debug)
			debug=1
			shift;
			;;
		-h|--hostname)
			docker_run_options+=("-h" "$2");
			container_hostname="$2";
			shift 2;
			;;
		-e|--env)
			docker_run_options+=("-e" "$2");
			_testval="${2#domainname=}"
			if [ "$_testval" != "$2" ]; then
				container_domainname="$_testval"
			fi
			_testval="${2#nameserver1=}"
			if [ "$_testval" != "$2" ]; then
				container_nameserver1="$_testval"
			fi
			shift 2;
			;;
		-i|--image|--resultimagename)
			resultimage_name="$2";
			if ! image_name_is_available "$resultimage_name"; then
				exit 1
			fi
			shift 2;
			;;
		--name|--containername)
			docker_run_options+=("--name" "$2");
			container_name="$2";
			shift 2;
			;;
		--updateto)
			updateto="$2";
			shift 2;
			;;
		-v|--volume)
			docker_run_options+=("-v" "$2");
			shift 2;
			;;
		--help)
			usage;
			;;
		--)
			shift;
			break;
			;;
	esac
done

if [ -n "$debug" ]; then
	set -x
fi

image_name="$1"
if [ -z "$image_name" ]; then
	usage
fi
shift

if [ -z "$container_domainname" ]; then
	## don't inherit domainname of host by default for updates
	container_domainname="univention.unconfigured"
	echo "Configuring container default domainname=$container_domainname"
	docker_run_options+=("-e" "domainname=$container_domainname")
fi

if [ -z "$container_nameserver1" ]; then
	## inherit host values by default
	for var in nameserver1 nameserver2 nameserver3; do
		_testval=$(univention-config-registry get "$var")
		if [ -n "$_testval" ]; then
			echo "INFO: Configuring container $var=$_testval as on this host"
			docker_run_options+=("-e" "$var=$_testval")
		fi
	done
fi

## Boot the container
uuid=$(docker run \
	"${docker_run_options[@]}" \
	"$image_name" \
	/sbin/init)

if [ -n "$uuid" ]; then
	echo "UUID: $uuid"
else
	echo "ERROR: Start of container failed"
	exit 1
fi

ucr_settings=(
server/role?generic
)

docker exec "$uuid" univention-config-registry set "${ucr_settings[@]}"

eval "$(docker exec "$uuid" ucr shell \
	server/role version/version version/patchlevel)"

if [ -z "$updateto" ]; then
	updateto="nextminor"
fi

univention_updater_options=()
docker_exec_update_options=()
if [ -n "$updateto" ]; then
	case "$updateto" in
		nextminor)
			updateto="$version_version-$((version_patchlevel+1))"
			
			;;
	esac
	univention_updater_options+=("--updateto=$updateto" --noninteractive)
else
	docker_exec_update_options=(-it)
fi

if [ -z "$resultimage_name" ]; then
	case "$server_role" in
		domaincontroller_*)
			docker_image_configuration="${server_role#domaincontroller_}"
			;;
		memberserver)
			docker_image_configuration="member"
			;;
		basesystem)
			docker_image_configuration="base"
			;;
		''|generic)
			docker_image_configuration="generic"
			;;
		*)
			docker_image_configuration="unknown"
			;;
	esac

	resultimage_name="${docker_repository_user}/${docker_image_prefix}-${docker_image_configuration}-${docker_image_pkg_arch}:${updateto}"

	if ! image_name_is_available "$resultimage_name"; then
		docker rm -f "$uuid"
		exit 1
	fi
fi

## Wait for ((UCR committed resolv.conf) and (runlevel 2))
echo -n "INFO: Waiting for container runlevel 2 "
for ((i=0;i<30;i++)); do
	sleep 1
	echo -n "."
	read clvl rlvl < <(docker exec "$uuid" runlevel)
	if [ "$rlvl" = 2 ]; then
		break 2
	fi
done
if [ "$i" -eq 30 ]; then
	echo "Timeout"
	exit 1
else
	echo "Ok"
fi

dns_check=$(docker exec "$uuid" \
	bash -c 'i=0; until host "$(ucr get repository/online/server)" >/dev/null 2>&1; do sleep 1; i=$((i+1)); if [ $i -gt 10 ]; then echo fail; break; fi; done')

if [ "$dns_check" = fail ]; then
	eval "$(docker exec "$uuid" univention-config-registry shell \
			repository/online/server)"
	echo "ERROR: Container cannot resolve $repository_online_server"
	exit 1
fi

## Run the update
docker exec "${docker_exec_update_options[@]}" "$uuid" \
	/usr/share/univention-updater/univention-updater net "${univention_updater_options[@]}"

docker exec "$uuid" apt-get clean

## Halt the container processes
_tmpval=$(docker stop "$uuid")
## Suppress useless default output
if [ "$_tmpval" != "$uuid" ]; then
	echo "$_tmpval"
fi

docker commit "$uuid" "$resultimage_name" || exit 1

echo -e "The updated $docker_image_configuration image has been generated:"
echo -e "\t$resultimage_name"
echo
echo "A new container can now be started from this image by:"
echo -e "\tdocker run -d -h unassigned-hostname \\"
echo -e "\t\t-e domainname=univention.unconfigured $resultimage_name /sbin/init"
echo "Or by running"
echo -e "\t/usr/share/univention-docker-dev/scripts/run-docker-image.sh \\"
echo -e "\t\t$resultimage_name"

resultimage_basename="${resultimage_name%:*}"
resultimage_name_latest="$resultimage_basename:latest"
if [ "$resultimage_name_latest" != "$resultimage_name" ]; then
	if image_name_is_available "$resultimage_name_latest"; then
		docker tag "$resultimage_name" "$resultimage_name_latest"
	else
		echo 'This should be fixed before uploading!'
	fi
fi

## Remove the work container if not in debug mode
if [ -z "$debug" ]; then
	_tmpval=$(docker rm "$uuid")
	## Suppress useless default output
	if [ "$_tmpval" != "$uuid" ]; then
		echo "$_tmpval"
	fi
else
	echo -e "Now you may remove the temporary container by running:"
	echo -e "\tdocker rm $uuid"
fi
